@@ -49,20 +49,20 @@ struct _BluetoothSettingsWidget {
 
 	/* Pairing */
 	BluetoothAgent      *agent;
-	AdwDialog           *pairing_dialog;
+	GtkWindow           *pairing_dialog;
 	GHashTable          *pairing_devices; /* key=object-path, value=boolean */
 
 	/* Properties */
-	AdwDialog           *properties_dialog;
+	GtkWindow           *properties_dialog;
 	char                *selected_bdaddr;
 	char                *selected_name;
 	char                *selected_object_path;
 
 	/* Device section */
-        GtkWidget           *page;
 	GtkWidget           *device_list;
 	GtkAdjustment       *focus_adjustment;
 	GtkSizeGroup        *row_sizegroup;
+	GtkWidget           *device_stack;
 	GtkWidget           *device_spinner;
 	GHashTable          *connecting_devices; /* key=bdaddr, value=boolean */
 
@@ -71,6 +71,7 @@ struct _BluetoothSettingsWidget {
 	GHashTable          *devices_type; /* key=bdaddr, value=guint32 */
 
 	/* Sharing section */
+	GtkWidget           *explanation_group;
 	gboolean             has_console;
 	GDBusProxy          *session_proxy;
 };
@@ -91,6 +92,8 @@ static guint signals[LAST_SIGNAL] = { 0 };
 #define MOUSE_PREFS		"mouse"
 #define SOUND_PREFS		"sound"
 
+#define ICON_SIZE 128
+
 /* We'll try to connect to the device repeatedly for that
  * amount of time before we bail out */
 #define CONNECT_TIMEOUT 3.0
@@ -104,22 +107,23 @@ static guint signals[LAST_SIGNAL] = { 0 };
 #define GNOME_SESSION_DBUS_OBJECT    "/org/gnome/SessionManager"
 #define GNOME_SESSION_DBUS_INTERFACE "org.gnome.SessionManager"
 
+#define FILLER_PAGE                  "filler-page"
+#define DEVICES_PAGE                 "devices-page"
+
 enum {
-	CONNECTING_STATE_IDLE = 0,
-	CONNECTING_STATE_LOADING = 1
+	CONNECTING_NOTEBOOK_PAGE_SWITCH = 0,
+	CONNECTING_NOTEBOOK_PAGE_SPINNER = 1
 };
 
 static void
-set_connecting_state (BluetoothSettingsWidget *self,
-		     int                      state)
+set_connecting_page (BluetoothSettingsWidget *self,
+		     int                      page)
 {
-	if (state == CONNECTING_STATE_LOADING) {
-		gtk_widget_set_visible (GTK_WIDGET (WID ("connecting_spinner")), TRUE);
-		gtk_widget_set_sensitive (GTK_WIDGET (WID ("switch_connection")), FALSE);
-	} else {
-		gtk_widget_set_visible (GTK_WIDGET (WID ("connecting_spinner")), FALSE);
-		gtk_widget_set_sensitive (GTK_WIDGET (WID ("switch_connection")), TRUE);
-	}
+	if (page == CONNECTING_NOTEBOOK_PAGE_SPINNER)
+		gtk_spinner_start (GTK_SPINNER (WID ("connecting_spinner")));
+	gtk_notebook_set_current_page (GTK_NOTEBOOK (WID ("connecting_notebook")), page);
+	if (page == CONNECTING_NOTEBOOK_PAGE_SWITCH)
+		gtk_spinner_stop (GTK_SPINNER (WID ("connecting_spinner")));
 }
 
 static void
@@ -193,7 +197,7 @@ connect_done (GObject      *source_object,
 			gtk_switch_set_active (button, !data->state);
 			g_debug ("Connection failed to %s: %s", data->bdaddr, error->message);
 		}
-		set_connecting_state (self, CONNECTING_STATE_IDLE);
+		set_connecting_page (self, CONNECTING_NOTEBOOK_PAGE_SWITCH);
 	}
 
 	remove_connecting (self, data->bdaddr);
@@ -225,8 +229,13 @@ add_device_type (BluetoothSettingsWidget *self,
 static void
 setup_pairing_dialog (BluetoothSettingsWidget *self)
 {
-	g_clear_pointer (&self->pairing_dialog, adw_dialog_force_close);
-	self->pairing_dialog = ADW_DIALOG (bluetooth_pairing_dialog_new ());
+	GtkWidget *toplevel;
+
+	g_clear_pointer (&self->pairing_dialog, gtk_window_destroy);
+	self->pairing_dialog = GTK_WINDOW (bluetooth_pairing_dialog_new ());
+	toplevel = GTK_WIDGET (gtk_widget_get_native (GTK_WIDGET (self)));
+	gtk_window_set_transient_for (self->pairing_dialog, GTK_WINDOW (toplevel));
+	gtk_window_set_modal (self->pairing_dialog, TRUE);
 }
 
 static gboolean
@@ -313,17 +322,17 @@ get_icade_pincode (char **pin_display_str)
 }
 
 static void
-display_cb (AdwDialog *dialog,
+display_cb (GtkDialog *dialog,
 	    int        response,
 	    gpointer   user_data)
 {
 	BluetoothSettingsWidget *self = user_data;
 
-	g_clear_pointer (&self->pairing_dialog, adw_dialog_force_close);
+	g_clear_pointer (&self->pairing_dialog, gtk_window_destroy);
 }
 
 static void
-enter_pin_cb (AdwDialog *dialog,
+enter_pin_cb (GtkDialog *dialog,
 	      int        response,
 	      gpointer   user_data)
 {
@@ -344,7 +353,7 @@ enter_pin_cb (AdwDialog *dialog,
 						       g_variant_new ("(s)", pin));
 
 		if (bluetooth_pairing_dialog_get_mode (BLUETOOTH_PAIRING_DIALOG (self->pairing_dialog)) == BLUETOOTH_PAIRING_MODE_PIN_QUERY) {
-			g_clear_pointer (&self->pairing_dialog, adw_dialog_force_close);
+			g_clear_pointer (&self->pairing_dialog, gtk_window_destroy);
 			return;
 		}
 		bluetooth_pairing_dialog_set_mode (BLUETOOTH_PAIRING_DIALOG (self->pairing_dialog),
@@ -355,7 +364,7 @@ enter_pin_cb (AdwDialog *dialog,
 		g_dbus_method_invocation_return_dbus_error (invocation,
 							    "org.bluez.Error.Canceled",
 							    "User cancelled pairing");
-		g_clear_pointer (&self->pairing_dialog, adw_dialog_force_close);
+		g_clear_pointer (&self->pairing_dialog, gtk_window_destroy);
 		return;
 	}
 
@@ -365,7 +374,7 @@ enter_pin_cb (AdwDialog *dialog,
 }
 
 static void
-confirm_remote_pin_cb (AdwDialog *dialog,
+confirm_remote_pin_cb (GtkDialog *dialog,
 		       int        response,
 		       gpointer   user_data)
 {
@@ -389,7 +398,7 @@ confirm_remote_pin_cb (AdwDialog *dialog,
 		g_dbus_method_invocation_return_dbus_error (invocation, "org.bluez.Error.Rejected", "Pairing refused from settings panel");
 	}
 
-	g_clear_pointer (&self->pairing_dialog, adw_dialog_force_close);
+	g_clear_pointer (&self->pairing_dialog, gtk_window_destroy);
 }
 
 static void
@@ -483,7 +492,7 @@ pincode_callback (GDBusMethodInvocation *invocation,
 				  G_CALLBACK (confirm_remote_pin_cb), user_data);
 	}
 
-	adw_dialog_present (self->pairing_dialog, GTK_WIDGET (self));
+	gtk_window_present (self->pairing_dialog);
 }
 
 static void
@@ -504,7 +513,7 @@ cancel_setup_cb (GObject      *source_object,
 }
 
 static void
-display_passkey_or_pincode_cb (AdwDialog *dialog,
+display_passkey_or_pincode_cb (GtkDialog *dialog,
 			       int        response,
 			       gpointer   user_data)
 {
@@ -524,7 +533,7 @@ display_passkey_or_pincode_cb (AdwDialog *dialog,
 		g_assert_not_reached ();
 	}
 
-	g_clear_pointer (&self->pairing_dialog, adw_dialog_force_close);
+	g_clear_pointer (&self->pairing_dialog, gtk_window_destroy);
 }
 
 static void
@@ -558,7 +567,7 @@ display_passkey_callback (GDBusMethodInvocation *invocation,
 				"path", g_strdup (g_dbus_proxy_get_object_path (device)),
 				g_free);
 
-	adw_dialog_present (self->pairing_dialog, GTK_WIDGET (self));
+	gtk_window_present (self->pairing_dialog);
 }
 
 static void
@@ -622,7 +631,7 @@ display_pincode_callback (GDBusMethodInvocation *invocation,
 	g_object_set_data_full (G_OBJECT (self->pairing_dialog),
 				"path", g_strdup (g_dbus_proxy_get_object_path (device)),
 				g_free);
-	adw_dialog_present (self->pairing_dialog, GTK_WIDGET (self));
+	gtk_window_present (self->pairing_dialog);
 
 	g_dbus_method_invocation_return_value (invocation, NULL);
 }
@@ -647,12 +656,11 @@ cancel_callback (GDBusMethodInvocation *invocation,
 
 	g_debug ("cancel_callback ()");
 
-	g_clear_pointer (&self->pairing_dialog, adw_dialog_force_close);
+	g_clear_pointer (&self->pairing_dialog, gtk_window_destroy);
 
 	child = gtk_widget_get_first_child (self->device_list);
 	while (child) {
-		if (GTK_IS_LIST_BOX_ROW (child))
-			g_object_set (G_OBJECT (child), "pairing", FALSE, NULL);
+		g_object_set (G_OBJECT (child), "pairing", FALSE, NULL);
 		child = gtk_widget_get_next_sibling (child);
 	}
 
@@ -662,7 +670,7 @@ cancel_callback (GDBusMethodInvocation *invocation,
 }
 
 static void
-confirm_cb (AdwDialog *dialog,
+confirm_cb (GtkDialog *dialog,
 	    int        response,
 	    gpointer   user_data)
 {
@@ -677,7 +685,7 @@ confirm_cb (AdwDialog *dialog,
 							    "org.bluez.Error.Canceled",
 							    "User cancelled pairing");
 	}
-	g_clear_pointer (&self->pairing_dialog, adw_dialog_force_close);
+	g_clear_pointer (&self->pairing_dialog, gtk_window_destroy);
 }
 
 static void
@@ -704,7 +712,7 @@ confirm_callback (GDBusMethodInvocation *invocation,
 			  G_CALLBACK (confirm_cb), user_data);
 	g_object_set_data (G_OBJECT (self->pairing_dialog), "invocation", invocation);
 
-	adw_dialog_present (self->pairing_dialog, GTK_WIDGET (self));
+	gtk_window_present (self->pairing_dialog);
 }
 
 static void
@@ -727,11 +735,11 @@ authorize_callback (GDBusMethodInvocation *invocation,
 			  G_CALLBACK (confirm_cb), user_data);
 	g_object_set_data (G_OBJECT (self->pairing_dialog), "invocation", invocation);
 
-	adw_dialog_present (self->pairing_dialog, GTK_WIDGET (self));
+	gtk_window_present (self->pairing_dialog);
 }
 
 static void
-authorize_service_cb (AdwDialog *dialog,
+authorize_service_cb (GtkDialog *dialog,
 		      int        response,
 		      gpointer   user_data)
 {
@@ -752,7 +760,7 @@ authorize_service_cb (AdwDialog *dialog,
 		msg = g_strdup_printf ("Rejecting service auth (HID): not paired or trusted");
 		g_dbus_method_invocation_return_dbus_error (invocation, "org.bluez.Error.Rejected", msg);
 	}
-	g_clear_pointer (&self->pairing_dialog, adw_dialog_force_close);
+	g_clear_pointer (&self->pairing_dialog, gtk_window_destroy);
 }
 
 static void
@@ -810,7 +818,7 @@ authorize_service_callback (GDBusMethodInvocation *invocation,
 		g_object_set_data_full (G_OBJECT (invocation), "device", g_object_ref (device), g_object_unref);
 		g_object_set_data (G_OBJECT (self->pairing_dialog), "invocation", invocation);
 
-		adw_dialog_present (self->pairing_dialog, GTK_WIDGET (self));
+		gtk_window_present (self->pairing_dialog);
 	}
 }
 
@@ -878,9 +886,6 @@ connect_callback (GObject      *source_object,
 			data->timeout_id = g_timeout_add (500, connect_timeout_cb, data);
 			return;
 		}
-
-		set_connecting_state (data->self, CONNECTING_STATE_IDLE);
-
 		g_debug ("Failed to connect to device %s: %s", data->device, error->message);
 	}
 
@@ -921,7 +926,7 @@ create_callback (GObject      *source_object,
 		if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
 			return;
 
-		g_clear_pointer (&self->pairing_dialog, adw_dialog_force_close);
+		g_clear_pointer (&self->pairing_dialog, gtk_window_destroy);
 
 		turn_off_pairing (user_data, path);
 
@@ -950,7 +955,7 @@ create_callback (GObject      *source_object,
 		return;
 	}
 
-	g_clear_pointer (&self->pairing_dialog, adw_dialog_force_close);
+	g_clear_pointer (&self->pairing_dialog, gtk_window_destroy);
 
 	g_hash_table_remove (self->pairing_devices, path);
 
@@ -1075,7 +1080,7 @@ switch_connected_state_set (GtkSwitch               *button,
 					  data);
 
 	add_connecting (self, data->bdaddr);
-	set_connecting_state (self, CONNECTING_STATE_LOADING);
+	set_connecting_page (self, CONNECTING_NOTEBOOK_PAGE_SPINNER);
 
 	return TRUE;
 }
@@ -1109,21 +1114,18 @@ update_properties (BluetoothSettingsWidget *self,
 	self->selected_object_path = g_strdup (bluetooth_device_get_object_path (device));
 
 	/* Hide all the buttons now, and show them again if we need to */
-	gtk_widget_set_visible (WID ("keyboard_button_row"), FALSE);
-	gtk_widget_set_visible (WID ("sound_button_row"), FALSE);
-	gtk_widget_set_visible (WID ("mouse_button_row"), FALSE);
-	gtk_widget_set_visible (WID ("send_button_row"), FALSE);
-
-        /* Properties dialog title */
-        adw_dialog_set_title (ADW_DIALOG (WID ("properties_dialog")), alias);
+	gtk_widget_hide (WID ("keyboard_button"));
+	gtk_widget_hide (WID ("sound_button"));
+	gtk_widget_hide (WID ("mouse_button"));
+	gtk_widget_hide (WID ("send_button"));
 
 	/* Name */
-        gtk_label_set_label (GTK_LABEL (WID ("device_name")), alias);
+	gtk_window_set_title (GTK_WINDOW (self->properties_dialog), alias);
 	g_free (self->selected_name);
 	self->selected_name = alias;
 
 	/* Icon */
-        gtk_image_set_from_icon_name (GTK_IMAGE (WID ("device_icon")), icon);
+	gtk_image_set_from_icon_name (GTK_IMAGE (WID ("image")), icon);
 
 	/* Connection */
 	button = GTK_SWITCH (WID ("switch_connection"));
@@ -1131,41 +1133,41 @@ update_properties (BluetoothSettingsWidget *self,
 	gtk_switch_set_state (button, connected);
 	gtk_switch_set_active (button, connected);
 	if (is_connecting (self, bdaddr)) {
-		set_connecting_state (self, CONNECTING_STATE_LOADING);
+		set_connecting_page (self, CONNECTING_NOTEBOOK_PAGE_SPINNER);
 	} else {
-		set_connecting_state (self, CONNECTING_STATE_IDLE);
+		set_connecting_page (self, CONNECTING_NOTEBOOK_PAGE_SWITCH);
 	}
 
 	/* Paired */
-	adw_action_row_set_subtitle (ADW_ACTION_ROW (WID ("paired_row")),
-				     paired ? _("Yes") : _("No"));
+	gtk_label_set_text (GTK_LABEL (WID ("paired_label")),
+			    paired ? _("Yes") : _("No"));
 
 	/* UUIDs */
+	gtk_widget_set_sensitive (GTK_WIDGET (button), connectable);
 	if (uuids && g_strv_contains ((const char * const*) uuids, "OBEXObjectPush"))
-		gtk_widget_set_visible (WID ("send_button_row"), TRUE);
+		gtk_widget_show (WID ("send_button"));
 
 	/* Type */
-	adw_action_row_set_subtitle (ADW_ACTION_ROW (WID ("type_row")),
-				     bluetooth_type_to_string (type));
+	gtk_label_set_text (GTK_LABEL (WID ("type_label")), bluetooth_type_to_string (type));
 	switch (type) {
 	case BLUETOOTH_TYPE_KEYBOARD:
-		gtk_widget_set_visible (WID ("keyboard_button_row"), TRUE);
+		gtk_widget_show (WID ("keyboard_button"));
 		break;
 	case BLUETOOTH_TYPE_MOUSE:
 	case BLUETOOTH_TYPE_TABLET:
-		gtk_widget_set_visible (WID ("mouse_button_row"), TRUE);
+		gtk_widget_show (WID ("mouse_button"));
 		break;
 	case BLUETOOTH_TYPE_HEADSET:
 	case BLUETOOTH_TYPE_HEADPHONES:
 	case BLUETOOTH_TYPE_OTHER_AUDIO:
-		gtk_widget_set_visible (WID ("sound_button_row"), TRUE);
+		gtk_widget_show (WID ("sound_button"));
 	default:
 		/* others? */
 		;
 	}
 
 	/* Address */
-	adw_action_row_set_subtitle (ADW_ACTION_ROW (WID ("address_row")), bdaddr);
+	gtk_label_set_text (GTK_LABEL (WID ("address_label")), bdaddr);
 
 	g_free (self->selected_bdaddr);
 	self->selected_bdaddr = bdaddr;
@@ -1180,29 +1182,32 @@ switch_panel (BluetoothSettingsWidget *self,
 		       0, panel);
 }
 
-static void
-keyboard_callback (AdwButtonRow            *row,
+static gboolean
+keyboard_callback (GtkButton        *button,
 		   BluetoothSettingsWidget *self)
 {
 	switch_panel (self, KEYBOARD_PREFS);
+	return TRUE;
 }
 
-static void
-mouse_callback (AdwButtonRow            *row,
+static gboolean
+mouse_callback (GtkButton        *button,
 		BluetoothSettingsWidget *self)
 {
 	switch_panel (self, MOUSE_PREFS);
+	return TRUE;
 }
 
-static void
-sound_callback (AdwButtonRow            *row,
-                BluetoothSettingsWidget *self)
+static gboolean
+sound_callback (GtkButton        *button,
+		BluetoothSettingsWidget *self)
 {
 	switch_panel (self, SOUND_PREFS);
+	return TRUE;
 }
 
 static void
-send_callback (AdwButtonRow            *row,
+send_callback (GtkButton               *button,
 	       BluetoothSettingsWidget *self)
 {
 	g_autoptr(GError) error = NULL;
@@ -1220,7 +1225,7 @@ update_visibility (BluetoothSettingsWidget *self)
 
 	g_object_get (G_OBJECT (self->client), "default-adapter-name", &name, NULL);
 	if (name != NULL) {
-		g_autofree char *description = NULL;
+		g_autofree char *label = NULL;
 		g_autofree char *path = NULL;
 		g_autofree char *uri = NULL;
 
@@ -1230,11 +1235,10 @@ update_visibility (BluetoothSettingsWidget *self)
 		/* translators: first %s is the name of the computer, for example:
 		 * Visible as “Bastien Nocera’s Computer” followed by the
 		 * location of the Downloads folder.*/
-		description = g_strdup_printf (_("Visible as “%s” and available for Bluetooth file transfers. Transferred files are placed in the <a href=\"%s\">Downloads</a> folder."), name, uri);
-		adw_preferences_page_set_description (ADW_PREFERENCES_PAGE (self->page), description);
-	} else {
-                adw_preferences_page_set_description (ADW_PREFERENCES_PAGE (self->page), NULL);
-        }
+		label = g_strdup_printf (_("Visible as “%s” and available for Bluetooth file transfers. Transferred files are placed in the <a href=\"%s\">Downloads</a> folder."), name, uri);
+		adw_preferences_group_set_description (ADW_PREFERENCES_GROUP (self->explanation_group), label);
+	}
+	gtk_widget_set_visible (self->explanation_group, name != NULL);
 }
 
 static void
@@ -1284,43 +1288,45 @@ remove_selected_device (BluetoothSettingsWidget *self)
 }
 
 static void
-confirm_dialog_remove_cb (AdwAlertDialog *dialog,
-                          gchar     *response,
-                          gpointer   user_data)
+confirm_dialog_response_cb (GtkDialog *dialog,
+			    gint       response,
+			    gpointer   user_data)
 {
 	BluetoothSettingsWidget *self = user_data;
 
-	remove_selected_device (self);
-        adw_dialog_force_close (self->properties_dialog);
+	if (response == GTK_RESPONSE_ACCEPT) {
+		remove_selected_device (self);
+		gtk_widget_hide (GTK_WIDGET (self->properties_dialog));
+	}
+
+	gtk_window_destroy (GTK_WINDOW (dialog));
 }
 
 static void
 show_confirm_dialog (BluetoothSettingsWidget *self,
 		     const char *name)
 {
-	AdwDialog *dialog;
+	GtkWidget *dialog;
 
-	dialog = adw_alert_dialog_new (_("Forget Device?"), NULL);
+	dialog = gtk_message_dialog_new (GTK_WINDOW (self->properties_dialog),
+					 GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+					 GTK_MESSAGE_QUESTION, GTK_BUTTONS_NONE,
+					 _("Remove “%s” from the list of devices?"), name);
+	g_object_set (G_OBJECT (dialog), "secondary-text",
+		      _("If you remove the device, you will have to set it up again before next use."),
+		      NULL);
 
-        adw_alert_dialog_format_body (ADW_ALERT_DIALOG (dialog),
-                                      _("“%s” will be removed from your saved devices. "
-                                                 "You will have to set it up again to use it."), name);
+	gtk_dialog_add_button (GTK_DIALOG (dialog), _("_Cancel"), GTK_RESPONSE_CANCEL);
+	gtk_dialog_add_button (GTK_DIALOG (dialog), _("_Remove"), GTK_RESPONSE_ACCEPT);
 
+	g_signal_connect (dialog, "response", G_CALLBACK (confirm_dialog_response_cb), self);
 
-	adw_alert_dialog_add_responses (ADW_ALERT_DIALOG (dialog),
-                                        "cancel", _("_Cancel"),
-				        "forget", _("_Forget"),
-				        NULL);
-	adw_alert_dialog_set_response_appearance (ADW_ALERT_DIALOG (dialog), "forget", ADW_RESPONSE_DESTRUCTIVE);
-
-	g_signal_connect (dialog, "response::forget", G_CALLBACK (confirm_dialog_remove_cb), self);
-
-        adw_dialog_present (dialog, GTK_WIDGET (self));
+	gtk_widget_show (dialog);
 }
 
 static void
-delete_callback (AdwButtonRow              *row,
-		 BluetoothSettingsWidget   *self)
+delete_clicked (GtkButton               *button,
+		BluetoothSettingsWidget *self)
 {
 	show_confirm_dialog (self, self->selected_name);
 }
@@ -1415,7 +1421,8 @@ static void
 activate_row (BluetoothSettingsWidget *self,
               GtkListBoxRow    *row)
 {
-	AdwDialog *w;
+	GtkWindow *w;
+	GtkWidget *toplevel;
 	g_autoptr(BluetoothDevice) device = NULL;
 	gboolean paired, trusted, is_setup;
 
@@ -1427,15 +1434,13 @@ activate_row (BluetoothSettingsWidget *self,
 	is_setup = paired || trusted;
 
 	if (is_setup) {
-		GtkWidget *page;
-
 		update_properties (self, device);
 
-		page = WID ("preferences_page");
-		adw_preferences_page_scroll_to_top (ADW_PREFERENCES_PAGE (page));
-
 		w = self->properties_dialog;
-		adw_dialog_present (w, GTK_WIDGET (self));
+		toplevel = GTK_WIDGET (gtk_widget_get_native (GTK_WIDGET (self)));
+		gtk_window_set_transient_for (w, GTK_WINDOW (toplevel));
+		gtk_window_set_modal (w, TRUE);
+		gtk_window_present (w);
 	} else {
 		start_pairing (self, row);
 	}
@@ -1447,12 +1452,12 @@ add_device_section (BluetoothSettingsWidget *self)
 	/* Discoverable spinner */
 	self->device_spinner = WID ("device_spinner");
 	g_object_bind_property (G_OBJECT (self->client), "default-adapter-setup-mode",
-				G_OBJECT (self->device_spinner), "visible",
+				G_OBJECT (self->device_spinner), "spinning",
 				G_BINDING_SYNC_CREATE);
 
 	/* Discoverable label placeholder, the real name is set in update_visibility().
 	 * If you ever see this string during normal use, please file a bug. */
-	self->page = WID ("page");
+	self->explanation_group = WID ("explanation_group");
 	update_visibility (self);
 
 	self->device_list = WID ("device_list");
@@ -1461,6 +1466,8 @@ add_device_section (BluetoothSettingsWidget *self)
 				    (GtkListBoxSortFunc)device_sort_func, NULL, NULL);
 	g_signal_connect_swapped (self->device_list, "row-activated",
 				  G_CALLBACK (activate_row), self);
+
+	self->device_stack = WID ("device_stack");
 }
 
 static void
@@ -1529,6 +1536,10 @@ device_added_cb (BluetoothClient *client,
 	gtk_list_box_append (GTK_LIST_BOX (self->device_list), row);
 	gtk_size_group_add_widget (self->row_sizegroup, row);
 
+	gtk_stack_set_transition_type (GTK_STACK (self->device_stack),
+				       GTK_STACK_TRANSITION_TYPE_SLIDE_DOWN);
+	gtk_stack_set_visible_child_name (GTK_STACK (self->device_stack), DEVICES_PAGE);
+
 	g_signal_connect_object (G_OBJECT (device), "notify",
 				 G_CALLBACK (device_changed_cb), self, 0);
 }
@@ -1563,9 +1574,15 @@ device_removed_cb (BluetoothClient *client,
 		}
 	}
 
-        if (!found) {
-             g_debug ("Didn't find a row to remove for tree path %s", object_path);
-        }
+	if (found) {
+		if (gtk_widget_get_first_child (self->device_list) == NULL) {
+			gtk_stack_set_transition_type (GTK_STACK (self->device_stack),
+						       GTK_STACK_TRANSITION_TYPE_NONE);
+			gtk_stack_set_visible_child_name (GTK_STACK (self->device_stack), FILLER_PAGE);
+		}
+	} else {
+		g_debug ("Didn't find a row to remove for tree path %s", object_path);
+	}
 }
 
 static void
@@ -1587,20 +1604,32 @@ devices_coldplug (BluetoothSettingsWidget *self)
 static void
 setup_properties_dialog (BluetoothSettingsWidget *self)
 {
-	self->properties_dialog = ADW_DIALOG (WID ("properties_dialog"));
+	GtkStyleContext *context;
 
-	g_signal_connect (G_OBJECT (WID ("delete_button_row")), "activated",
-			  G_CALLBACK (delete_callback), self);
-	g_signal_connect (G_OBJECT (WID ("mouse_button_row")), "activated",
+	self->properties_dialog = g_object_new (GTK_TYPE_DIALOG, "use-header-bar", TRUE, NULL);
+	gtk_window_set_hide_on_close (self->properties_dialog, TRUE);
+	gtk_widget_set_size_request (GTK_WIDGET (self->properties_dialog), 380, -1);
+	gtk_window_set_resizable (self->properties_dialog, FALSE);
+	gtk_window_set_child (self->properties_dialog, WID ("properties_vbox"));
+
+	g_signal_connect (G_OBJECT (WID ("delete_button")), "clicked",
+			  G_CALLBACK (delete_clicked), self);
+	g_signal_connect (G_OBJECT (WID ("mouse_button")), "clicked",
 			  G_CALLBACK (mouse_callback), self);
-	g_signal_connect (G_OBJECT (WID ("keyboard_button_row")), "activated",
+	g_signal_connect (G_OBJECT (WID ("keyboard_button")), "clicked",
 			  G_CALLBACK (keyboard_callback), self);
-	g_signal_connect (G_OBJECT (WID ("sound_button_row")), "activated",
+	g_signal_connect (G_OBJECT (WID ("sound_button")), "clicked",
 			  G_CALLBACK (sound_callback), self);
-	g_signal_connect (G_OBJECT (WID ("send_button_row")), "activated",
+	g_signal_connect (G_OBJECT (WID ("send_button")), "clicked",
 			  G_CALLBACK (send_callback), self);
 	g_signal_connect (G_OBJECT (WID ("switch_connection")), "state-set",
 			  G_CALLBACK (switch_connected_state_set), self);
+
+	/* Styling */
+	gtk_image_set_pixel_size (GTK_IMAGE (WID ("image")), ICON_SIZE);
+
+	context = gtk_widget_get_style_context (WID ("delete_button"));
+	gtk_style_context_add_class (context, "destructive-action");
 }
 
 static void
@@ -1757,8 +1786,8 @@ bluetooth_settings_widget_finalize (GObject *object)
 	BluetoothSettingsWidget *self = BLUETOOTH_SETTINGS_WIDGET(object);
 
 	g_clear_object (&self->agent);
-	g_clear_pointer (&self->properties_dialog, adw_dialog_force_close);
-	g_clear_pointer (&self->pairing_dialog, adw_dialog_force_close);
+	g_clear_pointer (&self->properties_dialog, gtk_window_destroy);
+	g_clear_pointer (&self->pairing_dialog, gtk_window_destroy);
 	g_clear_object (&self->session_proxy);
 
 	obex_agent_down ();
